from shrug_lang.errors import WtfError


class BaseOp:
    @staticmethod
    def check_for_undefined(val1, val2):
        """Raise an error with correct message if either value is undefined"""
        if val1[0] is None and val2[0] is None:
            raise ValueError(f'Undefined values {val1[1]} and {val2[1]}')
        if val1[0] is None:
            raise ValueError(f'Undefined values {val1[1]}')
        if val2[0] is None:
            raise ValueError(f'Undefined values {val2[1]}')
        return True

    @staticmethod
    def check_non_numbers(val1, val2, operation_type: str = ''):
        """Raise an error if either value is not an int"""
        if isinstance(val1, str) and isinstance(val2, str):
            raise TypeError(
                f'Unsupported {operation_type} types: string and string')
        if isinstance(val1, str) and isinstance(val2, int):
            raise TypeError(
                f'Unsupported {operation_type} types: str and number')
        if isinstance(val1, int) and isinstance(val2, str):
            raise TypeError(
                f'Unsupported {operation_type} types: number and str')
        return True

    @staticmethod
    def unpack_values(val1, val2):
        """
        Raise an error with correct message if either value is undefined and
        return value part of the tuple
        """
        BaseOp.check_for_undefined(val1, val2)
        val1 = val1[0]
        val2 = val2[0]
        return val1, val2

    @staticmethod
    def check_matching_type(val1, val2, operation: str = ''):
        """Raise an error if types do not match"""
        if type(val1) != type(val2):
            raise TypeError(f'Both values must be of the same type for '
                            f'"{operation}" operation')
        return True


class MathOp(BaseOp):
    """Math operations (mostly) follow python rules"""
    @staticmethod
    def unpack_values(val1, val2):
        val1, val2 = BaseOp.unpack_values(val1, val2)
        if isinstance(val1, bool):
            val1 = int(val1)
        if isinstance(val2, bool):
            val2 = int(val2)
        return val1, val2

    @staticmethod
    def add(val1, val2):
        val1, val2 = MathOp.unpack_values(val1, val2)
        if isinstance(val1, str):
            if isinstance(val2, str):
                return val1 + val2
            if isinstance(val2, int):
                return val1 + str(val2)
            raise WtfError
        if isinstance(val1, int):
            if isinstance(val2, str):
                return str(val1) + val2
            if isinstance(val2, int):
                return val1 + val2
            raise WtfError
        raise WtfError

    @staticmethod
    def subtract(val1, val2):
        val1, val2 = MathOp.unpack_values(val1, val2)
        MathOp.check_non_numbers(val1, val2, 'subtraction')
        return val1 - val2

    @staticmethod
    def multiply(val1, val2):
        val1, val2 = MathOp.unpack_values(val1, val2)
        if isinstance(val1, str) and isinstance(val2, str):
            raise TypeError('Unsupported multiplication types: '
                            'string and string')
        return val1 * val2

    @staticmethod
    def divide(val1, val2):
        val1, val2 = MathOp.unpack_values(val1, val2)
        MathOp.check_non_numbers(val1, val2, 'division')
        return val1 // val2

    @staticmethod
    def modulus(val1, val2):
        val1, val2 = MathOp.unpack_values(val1, val2)
        MathOp.check_non_numbers(val1, val2, 'modulus')
        return val1 % val2


class CompOp(BaseOp):
    """Comparators follow python rules"""
    @staticmethod
    def eq(val1, val2):
        val1, val2 = CompOp.unpack_values(val1, val2)
        return val1 == val2

    @staticmethod
    def neq(val1, val2):
        val1, val2 = CompOp.unpack_values(val1, val2)
        return val1 != val2

    @staticmethod
    def gt(val1, val2):
        val1, val2 = CompOp.unpack_values(val1, val2)
        CompOp.check_matching_type(val1, val2, 'greater than')
        return val1 > val2

    @staticmethod
    def gte(val1, val2):
        val1, val2 = CompOp.unpack_values(val1, val2)
        CompOp.check_matching_type(val1, val2, 'greater than or equal')
        return val1 >= val2

    @staticmethod
    def lt(val1, val2):
        val1, val2 = CompOp.unpack_values(val1, val2)
        CompOp.check_matching_type(val1, val2, 'less than')
        return val1 < val2

    @staticmethod
    def lte(val1, val2):
        val1, val2 = CompOp.unpack_values(val1, val2)
        CompOp.check_matching_type(val1, val2, 'less than or equal')
        return val1 <= val2



from typing import List

from shrug_lang.shrug_token import Token, TokenType


class Tokenizer:
    @staticmethod
    def parse_line(line: str) -> List[Token]:
        if line.lstrip().startswith('#'):
            return [Token(TokenType.COMMENT)]
        indent_size = 0
        for c in line:
            if c == ' ':
                indent_size += 1
            else:
                break
        unparsed_tokens = filter(None, Tokenizer.join_strings(line.split(' ')))
        tokens = [Token(TokenType.INDENT, indent_size)]
        tokens += [Tokenizer.parse_token(unparsed)
                   for unparsed in unparsed_tokens]
        tokens.append(Token(TokenType.EOL))
        return tokens

    @staticmethod
    def join_strings(unparsed_tokens: List[str]) -> List[str]:
        """Join strings that have spaces in them"""
        new_unparsed = []
        reading_string = False
        current_string = []
        for unparsed_token in unparsed_tokens:
            if reading_string:
                if unparsed_token.endswith('"'):
                    reading_string = False
                    current_string.append(unparsed_token)
                    new_unparsed.append(' '.join(current_string))
                    current_string.clear()
                else:
                    current_string.append(unparsed_token)
            elif (unparsed_token.startswith('"') and
                  (len(unparsed_token) == 1 or
                   not unparsed_token.endswith('"'))):
                reading_string = True
                current_string.append(unparsed_token)
            else:
                new_unparsed.append(unparsed_token)
        if len(current_string):
            new_unparsed.append(' '.join(current_string))
        return new_unparsed

    @staticmethod
    def parse_token(unparsed_token: str) -> Token:
        """Get token from given string"""
        if unparsed_token == '¯\_(ツ)_/¯':
            return Token(TokenType.SHRUG)
        if (len(unparsed_token) >= 2 and
                unparsed_token.startswith('"') and
                unparsed_token.endswith('"')):
            return Token(TokenType.STRING, unparsed_token[1:-1])
        if unparsed_token == 'False':
            return Token(TokenType.BOOL, False)
        if unparsed_token == 'True':
            return Token(TokenType.BOOL, True)
        if unparsed_token.isalpha():
            return Token(TokenType.ID, unparsed_token)
        if unparsed_token.isnumeric():
            return Token(TokenType.NUMBER, int(unparsed_token))
        return Token(TokenType.INVALID, unparsed_token)



import unittest

from shrug_lang.operators import CompOp
from .utils import BaseTokenParserTestCase, TokenGenerator


class TestCompOperations(unittest.TestCase):
    def test_equality(self):
        self.assertTrue(CompOp.eq(('',), ('',)))
        self.assertTrue(CompOp.eq(('abc',), ('abc',)))
        self.assertTrue(CompOp.eq((1,), (1,)))
        self.assertTrue(CompOp.eq((True,), (True,)))
        self.assertTrue(CompOp.eq((True,), (1,)))

        self.assertFalse(CompOp.eq(('',), ('a',)))
        self.assertFalse(CompOp.eq((2,), (1,)))
        self.assertFalse(CompOp.eq((False,), (True,)))
        self.assertFalse(CompOp.eq((True,), (2,)))

    def test_inequality(self):
        self.assertFalse(CompOp.neq(('',), ('',)))
        self.assertFalse(CompOp.neq(('abc',), ('abc',)))
        self.assertFalse(CompOp.neq((1,), (1,)))
        self.assertFalse(CompOp.neq((True,), (True,)))
        self.assertFalse(CompOp.neq((True,), (1,)))

        self.assertTrue(CompOp.neq(('',), ('a',)))
        self.assertTrue(CompOp.neq((2,), (1,)))
        self.assertTrue(CompOp.neq((False,), (True,)))
        self.assertTrue(CompOp.neq((True,), (2,)))

    def test_gt(self):
        self.assertTrue(CompOp.gt(('b',), ('a',)))
        self.assertTrue(CompOp.gt((2,), (1,)))

        self.assertRaises(TypeError, CompOp.gt, 'a', 1)
        self.assertRaises(TypeError, CompOp.gt, 1, '')

        self.assertFalse(CompOp.gt(('a',), ('a',)))
        self.assertFalse(CompOp.gt((1,), (1,)))

        self.assertFalse(CompOp.gt(('a',), ('b',)))
        self.assertFalse(CompOp.gt((1,), (2,)))

    def test_gte(self):
        self.assertTrue(CompOp.gte(('b',), ('a',)))
        self.assertTrue(CompOp.gte((2,), (1,)))

        self.assertRaises(TypeError, CompOp.gte, 'a', 1)
        self.assertRaises(TypeError, CompOp.gte, 1, '')

        self.assertTrue(CompOp.gte(('a',), ('a',)))
        self.assertTrue(CompOp.gte((1,), (1,)))

        self.assertFalse(CompOp.gte(('a',), ('b',)))
        self.assertFalse(CompOp.gte((1,), (2,)))

    def test_lt(self):
        self.assertFalse(CompOp.lt(('b',), ('a',)))
        self.assertFalse(CompOp.lt((2,), (1,)))

        self.assertRaises(TypeError, CompOp.lt, 'a', 1)
        self.assertRaises(TypeError, CompOp.lt, 1, '')

        self.assertFalse(CompOp.lt(('a',), ('a',)))
        self.assertFalse(CompOp.lt((1,), (1,)))

        self.assertTrue(CompOp.lt(('a',), ('b',)))
        self.assertTrue(CompOp.lt((1,), (2,)))

    def test_lte(self):
        self.assertFalse(CompOp.lte(('b',), ('a',)))
        self.assertFalse(CompOp.lte((2,), (1,)))

        self.assertRaises(TypeError, CompOp.lte, 'a', 1)
        self.assertRaises(TypeError, CompOp.lte, 1, '')

        self.assertTrue(CompOp.lte(('a',), ('a',)))
        self.assertTrue(CompOp.lte((1,), (1,)))

        self.assertTrue(CompOp.lte(('a',), ('b',)))
        self.assertTrue(CompOp.lte((1,), (2,)))


class TestCompParser(BaseTokenParserTestCase):
    def test_var_var_eq(self):
        tokens = [TokenGenerator.get_id('a'), TokenGenerator.get_number(5),
                  TokenGenerator.get_eol(),
                  TokenGenerator.get_id('b'), TokenGenerator.get_number(5),
                  TokenGenerator.get_eol(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_id('a'), TokenGenerator.get_id('b'),
                  TokenGenerator.get_eol()]
        expected = [True]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_int_int_neq(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number(4), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number(4), TokenGenerator.get_eol()]
        expected = [False]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_int_int_gt(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number(4), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_number(2),
                  TokenGenerator.get_eol()]
        expected = [True]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_int_int_gte(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number(3), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number(4), TokenGenerator.get_eol()]
        expected = [False]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_str_str_lt(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number('abc'), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_number('abd'),
                  TokenGenerator.get_eol()]
        expected = [True]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_str_str_lte(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number('abc'), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number('abc'), TokenGenerator.get_eol()]
        expected = [True]
        self.assertEqual(expected, self.process_tokens(tokens))


if __name__ == '__main__':
    unittest.main()



from dataclasses import dataclass
from enum import Enum
from typing import Union


class TokenType(Enum):
    SHRUG = 1
    NUMBER = 2
    STRING = 3
    ID = 4
    EOL = 5
    INVALID = 6
    BOOL = 7
    INDENT = 8
    COMMENT = 9


@dataclass(init=False)
class Token:
    type: TokenType
    value: Union[int, str, None] = None

    def __init__(self, _type: TokenType, value: Union[int, str, None]=None):
        if not isinstance(_type, TokenType):
            raise TypeError('Token type must be of TokenType')
        must_have_value = {TokenType.NUMBER, TokenType.STRING, TokenType.ID,
                           TokenType.INVALID, TokenType.BOOL,
                           TokenType.INDENT}
        if _type in must_have_value and value is None:
            raise ValueError(f'Token type {_type} must have a value')
        self.type = _type
        self.value = value



import unittest

from shrug_lang.errors import TokenError
from .utils import BaseTokenParserTestCase, TokenGenerator


class TestBasicTokenParser(BaseTokenParserTestCase):
    """Token parser should give correct output based on series of tokens"""
    def test_empty_line(self):
        tokens = [TokenGenerator.get_eol()]
        expected = []
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_single_int_output(self):
        tokens = [TokenGenerator.get_number(5), TokenGenerator.get_eol()]
        expected = [5]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_single_string_output(self):
        tokens = [TokenGenerator.get_string('abc'), TokenGenerator.get_eol()]
        expected = ['abc']
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_output_unassigned(self):
        tokens = [TokenGenerator.get_id(), TokenGenerator.get_eol()]
        expected = []
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_assign_int_and_output(self):
        tokens = [TokenGenerator.get_id(), TokenGenerator.get_number(123),
                  TokenGenerator.get_eol(), TokenGenerator.get_id(),
                  TokenGenerator.get_eol()]
        expected = [123]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_assign_bool_and_output(self):
        tokens = [TokenGenerator.get_id(), TokenGenerator.get_bool(True),
                  TokenGenerator.get_eol(), TokenGenerator.get_id(),
                  TokenGenerator.get_eol()]
        expected = [True]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_assign_string_and_output(self):
        tokens = [TokenGenerator.get_id(), TokenGenerator.get_string('abc'),
                  TokenGenerator.get_eol(), TokenGenerator.get_id(),
                  TokenGenerator.get_eol()]
        expected = ['abc']
        self.assertEqual(expected, self.process_tokens(tokens))


class TestInvalidTokens(BaseTokenParserTestCase):
    """Token parser should raise exception when given invalid token"""
    def test_invalid_token(self):
        self.assertRaises(TokenError, self.token_parser.next_token,
                          TokenGenerator.get_invalid('invalid'))

    def test_incomplete_line1(self):
        self.token_parser.next_token(TokenGenerator.get_shrug())
        self.assertRaises(TokenError, self.token_parser.next_token,
                          TokenGenerator.get_eol())

    def test_incomplete_line2(self):
        self.token_parser.next_token(TokenGenerator.get_id())
        self.token_parser.next_token(TokenGenerator.get_shrug())
        self.assertRaises(TokenError, self.token_parser.next_token,
                          TokenGenerator.get_eol())

    def test_invalid_token_at_state(self):
        self.token_parser.next_token(TokenGenerator.get_string())
        self.assertRaises(TokenError, self.token_parser.next_token,
                          TokenGenerator.get_string())


if __name__ == '__main__':
    unittest.main()



import unittest

from .utils import BaseTokenParserTestCase, TokenGenerator

cond_shrugs = [TokenGenerator.get_shrug()] * 3


class TestCondParsing(BaseTokenParserTestCase):
    def test_simple_true_cond(self):
        tokens = [
            TokenGenerator.get_indent(), *cond_shrugs, TokenGenerator.get_bool(True), TokenGenerator.get_eol(),
            TokenGenerator.get_indent(1), TokenGenerator.get_number(5), TokenGenerator.get_eol(),
            TokenGenerator.get_indent(1), TokenGenerator.get_number(6), TokenGenerator.get_eol(),
            TokenGenerator.get_indent(0), TokenGenerator.get_number(7), TokenGenerator.get_eol()
        ]
        processed = self.process_tokens(tokens)
        expected = [5, 6, 7]
        self.assertEqual(expected, processed)

    def test_simple_false_cond(self):
        tokens = [
            TokenGenerator.get_indent(), *cond_shrugs, TokenGenerator.get_bool(False), TokenGenerator.get_eol(),
            TokenGenerator.get_indent(1), TokenGenerator.get_number(5), TokenGenerator.get_eol(),
            TokenGenerator.get_indent(1), TokenGenerator.get_number(6), TokenGenerator.get_eol(),
            TokenGenerator.get_indent(0), TokenGenerator.get_number(7), TokenGenerator.get_eol()
        ]
        processed = self.process_tokens(tokens)
        expected = [7]
        self.assertEqual(expected, processed)

    def test_multiple_cond(self):
        tokens = [
            TokenGenerator.get_indent(), *cond_shrugs, TokenGenerator.get_bool(True), TokenGenerator.get_eol(),
            TokenGenerator.get_indent(1), TokenGenerator.get_number(5), TokenGenerator.get_eol(),
            TokenGenerator.get_indent(), *cond_shrugs, TokenGenerator.get_bool(True), TokenGenerator.get_eol(),
            TokenGenerator.get_indent(1), TokenGenerator.get_number(6), TokenGenerator.get_eol()
        ]
        processed = self.process_tokens(tokens)
        expected = [5, 6]
        self.assertEqual(expected, processed)

    def test_nested_cond(self):
        tokens = [
            TokenGenerator.get_indent(), *cond_shrugs, TokenGenerator.get_bool(True), TokenGenerator.get_eol(),
            TokenGenerator.get_indent(1), TokenGenerator.get_number(5), TokenGenerator.get_eol(),
            TokenGenerator.get_indent(1), *cond_shrugs, TokenGenerator.get_bool(True), TokenGenerator.get_eol(),
            TokenGenerator.get_indent(2), TokenGenerator.get_number(6), TokenGenerator.get_eol(),
            TokenGenerator.get_indent(2), *cond_shrugs, TokenGenerator.get_bool(True), TokenGenerator.get_eol(),
            TokenGenerator.get_indent(3), TokenGenerator.get_number(7), TokenGenerator.get_eol(),
            TokenGenerator.get_indent(1), *cond_shrugs, TokenGenerator.get_bool(False), TokenGenerator.get_eol(),
            TokenGenerator.get_indent(2), TokenGenerator.get_number(8), TokenGenerator.get_eol(),
        ]
        processed = self.process_tokens(tokens)
        expected = [5, 6, 7]
        self.assertEqual(expected, processed)


if __name__ == '__main__':
    unittest.main()



from enum import Enum

from shrug_lang.errors import TokenError
from shrug_lang.shrug_token import Token, TokenType


class ParserState(Enum):
    EMPTY = 0
    INVALID = 1001
    END = 1002

    MATH = 1
    ADDITION = 2
    SUBTRACTION = 3
    MULTIPLICATION = 4
    DIVISION = 5
    MODULUS = 6

    COMPARE = 11
    EQ = 12
    NEQ = 13
    GT = 14
    GTE = 15
    LT = 16
    LTE = 17

    COND = 21


class StateTransformer:
    """Calculates next state given a token"""

    def __init__(self):
        self.state_transformer = {
            ParserState.EMPTY: {
                TokenType.ID: ParserState.EMPTY,
                TokenType.NUMBER: ParserState.END,
                TokenType.BOOL: ParserState.END,
                TokenType.STRING: ParserState.END,
                TokenType.SHRUG: ParserState.MATH,
                TokenType.INDENT: ParserState.EMPTY,
                TokenType.COMMENT: ParserState.EMPTY,
            },

            ParserState.INVALID: {},

            ParserState.END: {},

            ParserState.MATH: {
                TokenType.ID: ParserState.ADDITION,
                TokenType.NUMBER: ParserState.ADDITION,
                TokenType.BOOL: ParserState.ADDITION,
                TokenType.STRING: ParserState.ADDITION,
                TokenType.SHRUG: ParserState.COMPARE
            },

            ParserState.ADDITION: {
                TokenType.ID: ParserState.END,
                TokenType.NUMBER: ParserState.END,
                TokenType.BOOL: ParserState.END,
                TokenType.STRING: ParserState.END,
                TokenType.SHRUG: ParserState.SUBTRACTION
            },

            ParserState.SUBTRACTION: {
                TokenType.ID: ParserState.END,
                TokenType.NUMBER: ParserState.END,
                TokenType.BOOL: ParserState.END,
                TokenType.STRING: ParserState.END,
                TokenType.SHRUG: ParserState.MULTIPLICATION
            },

            ParserState.MULTIPLICATION: {
                TokenType.ID: ParserState.END,
                TokenType.NUMBER: ParserState.END,
                TokenType.BOOL: ParserState.END,
                TokenType.STRING: ParserState.END,
                TokenType.SHRUG: ParserState.DIVISION
            },

            ParserState.DIVISION: {
                TokenType.ID: ParserState.END,
                TokenType.NUMBER: ParserState.END,
                TokenType.BOOL: ParserState.END,
                TokenType.STRING: ParserState.END,
                TokenType.SHRUG: ParserState.MODULUS
            },

            ParserState.MODULUS: {
                TokenType.ID: ParserState.END,
                TokenType.NUMBER: ParserState.END,
                TokenType.BOOL: ParserState.END,
                TokenType.STRING: ParserState.END
            },

            ParserState.COMPARE: {
                TokenType.ID: ParserState.EQ,
                TokenType.NUMBER: ParserState.EQ,
                TokenType.BOOL: ParserState.EQ,
                TokenType.STRING: ParserState.EQ,
                TokenType.SHRUG: ParserState.COND
            },

            ParserState.EQ: {
                TokenType.ID: ParserState.END,
                TokenType.NUMBER: ParserState.END,
                TokenType.BOOL: ParserState.END,
                TokenType.STRING: ParserState.END,
                TokenType.SHRUG: ParserState.NEQ
            },

            ParserState.NEQ: {
                TokenType.ID: ParserState.END,
                TokenType.NUMBER: ParserState.END,
                TokenType.BOOL: ParserState.END,
                TokenType.STRING: ParserState.END,
                TokenType.SHRUG: ParserState.GT
            },

            ParserState.GT: {
                TokenType.ID: ParserState.END,
                TokenType.NUMBER: ParserState.END,
                TokenType.BOOL: ParserState.END,
                TokenType.STRING: ParserState.END,
                TokenType.SHRUG: ParserState.GTE
            },

            ParserState.GTE: {
                TokenType.ID: ParserState.END,
                TokenType.NUMBER: ParserState.END,
                TokenType.BOOL: ParserState.END,
                TokenType.STRING: ParserState.END,
                TokenType.SHRUG: ParserState.LT
            },

            ParserState.LT: {
                TokenType.ID: ParserState.END,
                TokenType.NUMBER: ParserState.END,
                TokenType.BOOL: ParserState.END,
                TokenType.STRING: ParserState.END,
                TokenType.SHRUG: ParserState.LTE
            },

            ParserState.LTE: {
                TokenType.ID: ParserState.END,
                TokenType.NUMBER: ParserState.END,
                TokenType.BOOL: ParserState.END,
                TokenType.STRING: ParserState.END
            },

            ParserState.COND: {
                TokenType.ID: ParserState.END,
                TokenType.NUMBER: ParserState.END,
                TokenType.BOOL: ParserState.END,
                TokenType.STRING: ParserState.END,
                TokenType.SHRUG: ParserState.MATH
            }
        }

    def next_state(self, state: ParserState, token: Token) -> ParserState:
        try:
            return self.state_transformer[state][token.type]
        except KeyError:
            raise TokenError(f'Unexpected token {token} at state {state}')



import unittest

from shrug_lang.shrug_token import Token, TokenType


class TestToken(unittest.TestCase):
    """Test token creation"""
    def test_invalid_type(self):
        self.assertRaises(TypeError, Token, 'type')
        self.assertRaises(TypeError, Token, 1)

    def test_mandatory_values(self):
        self.assertRaises(ValueError, Token, TokenType.STRING)
        self.assertRaises(ValueError, Token, TokenType.NUMBER)
        self.assertRaises(ValueError, Token, TokenType.ID)
        self.assertRaises(ValueError, Token, TokenType.INVALID)

    def test_valid_tokens(self):
        Token(TokenType.SHRUG)
        Token(TokenType.NUMBER, 1)
        Token(TokenType.STRING, 'abc')
        Token(TokenType.ID, 'a')
        Token(TokenType.EOL)
        Token(TokenType.INVALID, 'invalid')


if __name__ == '__main__':
    unittest.main()



import unittest

from .utils import BaseTokenParserTestCase, TokenGenerator


class TestMathOperations(BaseTokenParserTestCase):
    """Integration tests for parsing lines that involve math operations"""

    """ADDITION"""

    def test_var_var_addition(self):
        tokens = [TokenGenerator.get_id('aa'), TokenGenerator.get_number(5),
                  TokenGenerator.get_eol(),
                  TokenGenerator.get_id('bb'), TokenGenerator.get_number(6),
                  TokenGenerator.get_eol(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_id('aa'),
                  TokenGenerator.get_id('bb'), TokenGenerator.get_eol()]
        expected = [11]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_str_str_addition(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_string('ab'),
                  TokenGenerator.get_string('cd'), TokenGenerator.get_eol()]
        expected = ['abcd']
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_int_int_addition(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_number(12),
                  TokenGenerator.get_number(3), TokenGenerator.get_eol()]
        expected = [15]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_str_int_addition(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_number(12),
                  TokenGenerator.get_string('abc'), TokenGenerator.get_eol()]
        expected = ['12abc']
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_int_str_addition(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_string('abc'),
                  TokenGenerator.get_number(34), TokenGenerator.get_eol()]
        expected = ['abc34']
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_assign_addition(self):
        tokens = [TokenGenerator.get_id('a'),
                  TokenGenerator.get_shrug(), TokenGenerator.get_number(12),
                  TokenGenerator.get_number(3), TokenGenerator.get_eol(),
                  TokenGenerator.get_id('a'), TokenGenerator.get_eol()]
        expected = [15]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_bool_bool_addition(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_bool(True),
                  TokenGenerator.get_bool(True), TokenGenerator.get_eol()]
        expected = [2]
        self.assertEqual(expected, self.process_tokens(tokens))

    """SUBTRACTION"""

    def test_int_int_subtraction(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_number(12),
                  TokenGenerator.get_shrug(), TokenGenerator.get_number(3),
                  TokenGenerator.get_eol()]
        expected = [9]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_str_str_subtraction(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_string('ab'),
                  TokenGenerator.get_shrug(), TokenGenerator.get_string('cd'),
                  TokenGenerator.get_eol()]
        self.assertRaises(TypeError, self.process_tokens, tokens)

    def test_str_int_subtraction(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_number(12),
                  TokenGenerator.get_shrug(), TokenGenerator.get_string('abc'),
                  TokenGenerator.get_eol()]
        self.assertRaises(TypeError, self.process_tokens, tokens)

    def test_int_str_subtraction(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_string('abc'),
                  TokenGenerator.get_shrug(), TokenGenerator.get_number(34),
                  TokenGenerator.get_eol()]
        self.assertRaises(TypeError, self.process_tokens, tokens)

    def test_assign_subtraction(self):
        tokens = [TokenGenerator.get_id('a'), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number(12), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number(3), TokenGenerator.get_eol(),
                  TokenGenerator.get_id('a'), TokenGenerator.get_eol()]
        expected = [9]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_bool_bool_subtraction(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_bool(False),
                  TokenGenerator.get_shrug(), TokenGenerator.get_bool(True),
                  TokenGenerator.get_eol()]
        expected = [-1]
        self.assertEqual(expected, self.process_tokens(tokens))

    """MULTIPLICATION"""

    def test_int_int_multiplication(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_number(13),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number(4), TokenGenerator.get_eol()]
        expected = [52]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_str_str_multiplication(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_string('ab'),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_string('cd'), TokenGenerator.get_eol()]
        self.assertRaises(TypeError, self.process_tokens, tokens)

    def test_str_int_multiplication(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_number(3),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_string('abc'), TokenGenerator.get_eol()]
        expected = ['abcabcabc']
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_int_str_multiplication(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_string('abc'),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number(2), TokenGenerator.get_eol()]
        expected = ['abcabc']
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_assign_multiplication(self):
        tokens = [TokenGenerator.get_id('a'), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number(12), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_number(3),
                  TokenGenerator.get_eol(), TokenGenerator.get_id('a'),
                  TokenGenerator.get_eol()]
        expected = [36]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_bool_bool_multiplication(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_bool(False),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_bool(True), TokenGenerator.get_eol()]
        expected = [0]
        self.assertEqual(expected, self.process_tokens(tokens))

    """DIVISION"""

    def test_int_int_division(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_number(12),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_number(4),
                  TokenGenerator.get_eol()]
        expected = [3]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_str_str_division(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_string('ab'),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_string('cd'),
                  TokenGenerator.get_eol()]
        self.assertRaises(TypeError, self.process_tokens, tokens)

    def test_str_int_division(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_number(3),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_string('abc'),
                  TokenGenerator.get_eol()]
        self.assertRaises(TypeError, self.process_tokens, tokens)

    def test_int_str_division(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_string('abc'),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_number(2),
                  TokenGenerator.get_eol()]
        self.assertRaises(TypeError, self.process_tokens, tokens)

    def test_assign_division(self):
        tokens = [TokenGenerator.get_id('a'), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number(14), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number(3), TokenGenerator.get_eol(),
                  TokenGenerator.get_id('a'), TokenGenerator.get_eol()]
        expected = [4]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_bool_bool_division(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_bool(True),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_bool(True),
                  TokenGenerator.get_eol()]
        expected = [1]
        self.assertEqual(expected, self.process_tokens(tokens))

    """MODULUS"""

    def test_int_int_modulus(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_number(12),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number(5), TokenGenerator.get_eol()]
        expected = [2]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_str_str_modulus(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_string('ab'),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_string('cd'), TokenGenerator.get_eol()]
        self.assertRaises(TypeError, self.process_tokens, tokens)

    def test_str_int_modulus(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_number(3),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_string('abc'), TokenGenerator.get_eol()]
        self.assertRaises(TypeError, self.process_tokens, tokens)

    def test_int_str_modulus(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_string('abc'),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number(2), TokenGenerator.get_eol()]
        self.assertRaises(TypeError, self.process_tokens, tokens)

    def test_assign_modulus(self):
        tokens = [TokenGenerator.get_id('a'), TokenGenerator.get_shrug(),
                  TokenGenerator.get_number(10), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_number(3),
                  TokenGenerator.get_eol(), TokenGenerator.get_id('a'),
                  TokenGenerator.get_eol()]
        expected = [1]
        self.assertEqual(expected, self.process_tokens(tokens))

    def test_bool_bool_modulus(self):
        tokens = [TokenGenerator.get_shrug(), TokenGenerator.get_bool(True),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_shrug(), TokenGenerator.get_shrug(),
                  TokenGenerator.get_bool(True), TokenGenerator.get_eol()]
        expected = [0]
        self.assertEqual(expected, self.process_tokens(tokens))


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
from setuptools import setup, find_packages


def read_file(name):
    with open(name) as f:
        return f.read().strip()


install_requires = [
    'dataclasses',
]

setup(
    name='shrug-lang',
    version='0.0.3',
    description='Interpreter for the Shrug Programming Language',
    long_description=read_file('README.md'),
    long_description_content_type='text/markdown',
    maintainer='Ben Wu',
    maintainer_email='bwub124@gmail.com',
    url='https://github.com/Ben-Wu/ShrugProgrammingLanguage',
    packages=find_packages(),
    include_package_data=True,
    install_requires=install_requires,
    entry_points="""
        [console_scripts]
        shruglang=shrug_lang.interpreter:start_interpreter
    """,
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3 :: Only',
        'Development Status :: 3 - Alpha',
    ]
)



from argparse import ArgumentParser
from sys import stderr

from shrug_lang.token_parser import TokenError, TokenParser
from shrug_lang.tokenizer import Tokenizer

# ¯\_(ツ)_/¯

DESCRIPTION = """
Interpreter for Shrug language
"""


def parse_args():
    parser = ArgumentParser(description=DESCRIPTION)
    parser.add_argument('-q', '--quiet', action='store_true')
    return parser.parse_args()


def start_interpreter():
    args = parse_args()
    parser = TokenParser()
    try:
        while True:
            if not args.quiet:
                print('>> ', end='')
            line = input()
            tokens = Tokenizer.parse_line(line)
            try:
                for token in tokens:
                    val = parser.next_token(token)
                    if val is not None:
                        print(val)
            except (TokenError, ValueError, TypeError, ZeroDivisionError) as e:
                print(f'{type(e).__name__}: {e}', file=stderr)
    except (EOFError, KeyboardInterrupt):
        print()


if __name__ == '__main__':
    start_interpreter()
